#include "tsys.h"
#include "notify.h"

#define NOTIFY_EXECUTE_FUNC (-1)

struct notify {
	long event; 

	struct tobj *execute;
	long method;
	long data;

	struct notify *next;
};

int notify_add( struct tobj *cause, long event,
		struct tobj *execute, long method ,long data)
{
	struct notify *n;
	
	if (( n = malloc( sizeof(struct notify) ) )){
		n->event = event;
		n->execute = execute;
		n->method = method;
		n->data = data;
	
		n->next = cause->notifies;
		cause->notifies = n;

		return 1;
	} else 
		return 0;
}
int notify_add_fnc(struct tobj *cause, long event,
		notifyfnc fnc, long data)
{
	return notify_add( cause , event, (struct tobj*)NOTIFY_EXECUTE_FUNC ,
			TPDATA( fnc ), data);
}

void notify_del3( struct tobj *cause )
{
	struct notify *n;

	while( cause->notifies ){
		n = cause->notifies->next;
		free( cause->notifies );
		cause->notifies = n;
	}
}

void notify_del2( struct tobj *cause , long event )
{
	struct notify *n, *prev;
	for( n = cause->notifies, prev=NULL ; n ; n=n->next ){
		if ( n->event == event ){
			if (prev)
				prev->next = n->next;
			else
				cause->notifies->next = n->next;
			
			free( n );
		}
		prev = n;
	}
}

void notify_del( struct tobj *cause, long event,
		struct tobj *execute, long method )
{
	struct notify *n, *prev;
	for( n = cause->notifies, prev=NULL ; n ; n=n->next ){
		if ( n->event == event &&
				n->execute == execute && n->method == method ){
			if (prev)
				prev->next = n->next;
			else
				cause->notifies->next = n->next;
			
			free( n );
		}
		prev = n;
	}
}

void notify_del_fnc(struct tobj *cause, long event,
		notifyfnc fnc )
{
	notify_del( cause , event , (struct tobj*)NOTIFY_EXECUTE_FUNC ,
			TPDATA(fnc) );
}

int notify_process( struct tobj *cause , long event )
{
	struct notify *n;
	int ret = 0;
	
	for(n = cause->notifies ; n ; n=n->next) {
		if ( n->event == event ){
			if (n->execute == (struct tobj*)NOTIFY_EXECUTE_FUNC){
				notifyfnc fnc;
				fnc = (notifyfnc)n->method;
				(*fnc)( cause , n->data );
			}
			else if (n->method == NOTIFY_SEND_EVENT )
				tevent_send( cause , n->execute , n->data );
			else
				tobj_domethod( n->execute, n->method, n->data );
			ret=1;
		}
	}
	
	return ret;
}
