#define _MTAB_
#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and methods */
#include "tcmenu.h"
#include "tlist.h"
#include <string.h>

#include "prvclass/tcmenu.h"


/* ex.
struct tcmenu_node cmenu[] = {
	{"Open..." , tcmenu_dupa , 0},
	{"Save..." , tcmenu_dupa , 0},
	{ TMENU_SEP , 0 , 0 },
	{"Quit..." , tcmenu_dupa , 0},
	{ TMENU_END , 0 , 0 }
};
*/
struct tcmenu_entry mtab[] = {{ TMENU_END , NULL , 0 }};

int add_menuobj(struct tobj *menu, char * title, notifyfnc fnc, long data)
{
	struct tcmenu_entry entry;
	entry.title = title;
	entry.fnc = fnc;
	entry.data = data;
	if (!tobj_domethod(menu, TMENU_ENTRY_ADD, TPDATA(&entry)))
		return 0;
	return 1;
}

static int tcmenu_entry_add(struct tobj *obj, struct tcmenu_entry *entry)
{
	struct tcmenu *cmenu;
	struct tcmenu_fentry *tmp;

	cmenu = obj->data;

	if ((tmp =
	     realloc(cmenu->fentries,
		     (cmenu->num + 1)* sizeof(struct tcmenu_fentry)))) {
		cmenu->fentries = tmp;
		cmenu->num++;

		(cmenu->fentries[cmenu->num - 1]).fnc = entry->fnc;
		(cmenu->fentries[cmenu->num - 1]).data = entry->data;

		tobj_domethod( obj, TLIST_ENTRY_ADD , TPDATA( entry->title ) );
		
		return 1;
	} else
		return 0;

}

int tcmenu_domethod(struct tobj *obj, long method, long data)
{
	struct tcmenu *cmenu;
	cmenu = (struct tcmenu *) obj->data;

	switch (method) {
	case TMENU_ENTRY_ADD:
		if (!(tcmenu_entry_add( obj , (struct tcmenu_entry*) data)))
			tsys_error_exit
			    ("ERROR: While changing wordfile in tcmenu.\n");
		return 1;
	default:
		return (*cmenu->list_fnc.domethod)(obj, method, data);
	}
	return 0;
}
static void tcmenu_do_fnc(struct tobj *obj)
{
	struct tcmenu *cmenu;
	unsigned long i;
	cmenu = (struct tcmenu *) obj->data;

	i = txget( obj , TLIST_ACTIVE );
	if (i < cmenu->num && *(cmenu->fentries[i]).fnc)
		(*(cmenu->fentries[i]).fnc)( obj , (cmenu->fentries[i]).data );

	/* Change active object to next one */
	tobj_domethod(obj->parent, TWIN_ACTIVATE_NEXT, 0);

	tevent_send(obj, EVENT_TYPE_USER, EVENT_TMENU_ACCEPT);
}

static void tcmenu_reject(struct tobj *obj)
{
	/* Change active object to next one */
	tobj_domethod(obj->parent, TWIN_ACTIVATE_NEXT, 0);

	tevent_send(obj, EVENT_TYPE_USER, EVENT_TMENU_REJECT);
}

int tcmenu_doevent(struct tobj *obj, int keycode)
{
	struct tcmenu *cmenu;
	cmenu = (struct tcmenu *) obj->data;

	switch (keycode) {
	case K_RET:
		tcmenu_do_fnc(obj);
		break;
	case K_ESC:
		tcmenu_reject(obj);
		break;
	default:
		return  (*cmenu->list_fnc.doevent) (obj, keycode);
	}
	return 1;
}

void tcmenu_free(struct tobj *obj)
{
	struct tcmenu *cmenu;
	cmenu = (struct tcmenu *) obj->data;

	if (cmenu)
		free(cmenu);
}
void tcmenu_cleanup(struct tobj *obj)
{
	struct tcmenu *cmenu;
	cmenu = (struct tcmenu *) obj->data;

	tlist_cleanup( obj );
	if (cmenu->fentries)
		free( cmenu->fentries );
}

int tcmenu_setup(struct tobj *obj, struct tobj_fnc *fnc,
		struct tcmenu_entry *cmenutab)
{
	struct tcmenu *cmenu;
	unsigned long i, len=0;
	char **texttab;
	char nomem[] = "ERROR: While allocating memory for tcmenu.\n";
	cmenu = (struct tcmenu*)obj->data;

	while( cmenutab[len].title != TMENU_END )
		len++;
	
	if (!( texttab = malloc( sizeof(char*)*(len + 1) )  )){
		tsys_error(nomem);
		return 0;
	} else if (len && !( cmenu->fentries = malloc(
				sizeof(struct tcmenu_fentry)*(len) )  )){
		tsys_error(nomem);
		return 0;
	} else {
		for( i=0 ; i < len ; i++ ){
			texttab[i]=cmenutab[i].title;
			(cmenu->fentries[i]).fnc = cmenutab[i].fnc;
			(cmenu->fentries[i]).data = cmenutab[i].data;
		}
		texttab[i] = NULL;
		cmenu->num = len;

		if (!(tlist_setup( obj, &cmenu->list_fnc, texttab)))
			return 0;

		if (len == 0)
			free(cmenu->fentries);

		free( texttab );

		fnc->dorefresh = cmenu->list_fnc.dorefresh;
		fnc->doevent = tcmenu_doevent;
		fnc->free = tcmenu_free;
		fnc->domethod = tcmenu_domethod;
		
		return 1;
	}
}

struct tobj *tcmenu_init(struct tobj *parent, unsigned int x,
			  unsigned int y, unsigned int w, unsigned h,
			  struct tcmenu_entry *cmenutab )
{
	struct tobj *obj;
	struct tcmenu *cmenu;
	char nomem[] = "ERROR: While allocating memory for tcmenu.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(cmenu = calloc(1, sizeof(struct tcmenu)))) {
		tsys_error(nomem);
		return 0;
	}
	else {
		obj->data = cmenu;
		obj->x = x;
		obj->y = y;
		obj->h = h;
		obj->w = w;

		/* setting default values */
                obj->fgcolor = TCMENUFG;
                obj->fgcolor2 = TCMENUFG2;
                obj->bgcolor = TCMENUBG;
                obj->bgcolor2 =TCMENUBG2;
		obj->activeable=1;

		if ((tcmenu_setup( obj , &(obj->fnc), cmenutab ) )){
			tobj_init( parent, obj );
			return obj;
		}
		else
			return 0;
	}
}

