#include "tsys.h"
#include "tobj.h"
#include "tfamily.h"
/* tobj real hide ;) */
void tobj_rhide(struct tobj *obj)
{
        tset(obj, TOBJ_HIDDEN, 1);
        tset(obj, TOBJ_ACTIVEABLE, 0);
	//tobj_domethod(obj, TOBJ_ACTIVATE, 0);
}

void tobj_show(struct tobj *obj)
{
	tset(obj, TOBJ_HIDDEN, 0);
	tset(obj, TOBJ_ACTIVEABLE, 1);
//	tobj_domethod(obj, TOBJ_SHOW, 1);
	tobj_domethod(obj, TOBJ_ACTIVATE, 1);
}

int tobj_isactive_inwin(struct tobj *obj)
{
	if (obj->parent) {
		return tfamily_isactivechild(obj->parent, obj);
	} else {
		return 1;
	}

}
int tobj_isactive(struct tobj *obj)
{
	if (obj->parent) {
		return tfamily_isactive(obj->parent, obj);
	} else {
		return 1;
	}

}
void tobj_activate(struct tobj *obj)
{
	if (obj && obj->parent) {
		if (obj->hidden)
			obj->hidden = 0;
		/* activating child in family */
		tfamily_activatechild(obj->parent, obj);

		tevent_send( obj, EVENT_TYPE_USER, EVENT_TOBJ_ACTIVE );
	}
}
void tobj_activate_prev(struct tobj *parent)
{
	if (parent) {
		/* activating next child in family */
		tfamily_activateprevchild(parent);

		if ( tfamily_activechild(parent) )
			tevent_send( tfamily_activechild(parent),
					EVENT_TYPE_USER, EVENT_TOBJ_ACTIVE );
	}
}

void tobj_activate_next(struct tobj *parent)
{
	if (parent) {
		/* activating next child in family */
		tfamily_activatenextchild(parent);

		if ( tfamily_activechild(parent) )
			tevent_send( tfamily_activechild(parent),
					EVENT_TYPE_USER, EVENT_TOBJ_ACTIVE );
	}
}
/* activate next, but if active child is only child which can be
 * activated, then don't activate any child */
void tobj_activate_next2(struct tobj *parent)
{
	struct tobj *obj, *obj2;
	if (parent) {
		obj = tfamily_activechild( parent );
		
		/* activating next child in family */
		tfamily_activatenextchild(parent);

		obj2 = tfamily_activechild( parent );
		if ( obj == obj2 )
			tfamily_activatechild( parent, NULL );
		else
			tevent_send( obj2,
				EVENT_TYPE_USER, EVENT_TOBJ_ACTIVE );
	}
}

/* get absolute value of obj position */
unsigned int tobj_getx(struct tobj *obj)
{
	if (obj) {
		return (obj->x + tobj_getx(obj->parent));
	} else
		return 0;
}
unsigned int tobj_gety(struct tobj *obj)
{
	if (obj) {
		return (obj->y + tobj_gety(obj->parent));
	} else
		return 0;
}
static void tobj_hide( struct tobj *obj , int hide )
{
	obj->hidden = hide;
	
	if (tobj_isactive_inwin(obj))
		tobj_activate_next2( obj->parent );

	tsystem->refresh = 1;
}

static int tobj_set(struct tobj *obj, struct tobj_set *tos)
{
	switch (tos->attr) {
	case TOBJ_LEFT:
		obj->x = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_TOP:
		obj->y = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_WIDTH:
		obj->w = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_HEIGHT:
		obj->h = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_FGCOLOR:
		obj->fgcolor = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_FGCOLOR2:
		obj->fgcolor2 = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_BGCOLOR:
		obj->bgcolor = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_BGCOLOR2:
		obj->bgcolor2 = tos->data;
		tsystem->refresh = 1;
		break;
	case TOBJ_HIDDEN:
		tobj_hide( obj , tos->data );
		break;
	case TOBJ_ACTIVEABLE:
		obj->activeable = tos->data;
		if (tobj_isactive_inwin(obj))
			tobj_activate_next2( obj->parent );
		break;
	default:
		if (obj->fnc.domethod)
			return (*obj->fnc.domethod) (obj, TOBJ_SET,
						      TPDATA(tos));
	}
	return 1;
}

static int tobj_get(struct tobj *obj, struct tobj_get *tog)
{
	switch (tog->attr) {
	case TOBJ_LEFT:
		*(tog->data) = obj->x;
		break;
	case TOBJ_TOP:
		*(tog->data) = obj->y;
		break;
	case TOBJ_WIDTH:
		*(tog->data) = obj->w;
		break;
	case TOBJ_HEIGHT:
		*(tog->data) = obj->h;
		break;
	case TOBJ_ACTIVEABLE:
		*(tog->data) = obj->activeable;
		break;
	case TOBJ_HIDDEN:
		*(tog->data) = obj->hidden;
		break;
	case TOBJ_FGCOLOR:
		*(tog->data) = obj->fgcolor;
		break;
	case TOBJ_FGCOLOR2:
		*(tog->data) = obj->fgcolor2;
		break;
	case TOBJ_BGCOLOR:
		*(tog->data) = obj->bgcolor;
		break;
	case TOBJ_NEXTOBJ:
		*(tog->data) = TPDATA(obj->next);
		break;
	case TOBJ_PARENTOBJ:
		*(tog->data) = TPDATA(obj->parent);
		break;
	case TOBJ_ACTIVE:
		*(tog->data) = tobj_isactive(obj);
		break;
	default:
		if (obj->fnc.domethod)
			return (*obj->fnc.domethod)(obj, TOBJ_GET,TPDATA(tog));
	}
	return 1;
}
int tobj_domethod(struct tobj *obj, long method, long data)
{
	switch (method) {
	case TOBJ_SET:
		return tobj_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return tobj_get(obj, (struct tobj_get *) data);
	case TOBJ_ACTIVATE:
		tobj_activate(obj);
		break;
	case TOBJ_SHOW:
		tobj_hide( obj , 0 );
		break;
	case TOBJ_HIDE:
		tobj_hide( obj , 1 );
		break;
	default:
		if (obj->fnc.domethod)
			return (*obj->fnc.domethod) (obj, method, data);
	}
	return 1;
}
int tobj_doevent(struct tobj *obj, int keycode)
{
	if (obj){
		if (obj->fnc.doevent && (*obj->fnc.doevent) (obj, keycode)) {
			/* check notifies */
			notify_process( obj , keycode );

			return 1;
		}
		else {
			/* check notifies */
			notify_process( obj , keycode );
			return 0;
		}
	} else return 0;
}
int tobj_init(struct tobj *parent, struct tobj *obj)
{
	if (parent) {
		/* adding to family */
		tfamily_addchild(parent, obj);

	} else { /* adding rootwindow */

		obj->next = NULL;
		obj->parent = NULL;
	}
	obj->notifies = NULL;
	obj->hidden = 0;
	tsystem->refresh = 1;
	return 1;

}
void tobj_free(struct tobj *obj)
{
	if (obj->fnc.free)
		(*obj->fnc.free) (obj);

	obj->fnc.free = NULL;
	obj->fnc.domethod = NULL;
	obj->fnc.doevent = NULL;
	obj->fnc.dorefresh = NULL;
	obj->garbage = 1;
	
	tsystem->garbage = 1;
	tsystem->refresh = 1;
}
void tobj_garbage(struct tobj *obj)
{
	if (obj->fnc.garbage)
		(*obj->fnc.garbage)(obj);

	if (obj->garbage){
		if (obj->parent) {
			/* removing from family */
			tfamily_delchild(obj->parent, obj);
		} else
			tsystem->rootwin = NULL;

		notify_del3( obj );

		free( obj );
	}
}
void tobj_dorefresh(struct tobj *obj)
{
	t_gotoxy(tobj_getx(obj), tobj_gety(obj));

	if ( obj->fnc.dorefresh && !obj->hidden ){
		(*obj->fnc.dorefresh) (obj);
		tsystem->sync = 1;
	}
}
