#ifndef _TOBJ_H
#define _TOBJ_H

#include "tcolors.h"
#include "notify.h"

struct tobj_fnc {
	/* DON'T CALL those function direcly */
	void (*dorefresh) (struct tobj *);
	void (*free) (struct tobj *);
	void (*garbage) (struct tobj *);
	int (*doevent) (struct tobj *, int keycode);
	int (*domethod) (struct tobj *, long method, long data);
};

struct tobj {
	unsigned int x, y;
	unsigned int w, h;

	char fgcolor;
	char fgcolor2;
	char bgcolor;
	char bgcolor2;
	char bgchar;

	int activeable;
	int hidden;

	struct tobj *next;

	struct tobj *parent;	/* or NULL for no parent */

	void *data;

	long userdata; /* here you can keep what you want */

	struct tobj_fnc fnc;

	struct notify *notifies;
	int garbage;
};

struct tobj_set {
	long attr;
	long data;
};

struct tobj_get {
	long attr;
	long *data;
};

/* Events (1101-1200) */
#define EVENT_TOBJ_ACTIVE (1101L)

/* defines of methods IDs (101-200)*/
#define TOBJ_SET (101L)
#define TOBJ_GET (102L)
#define TOBJ_ACTIVATE	(103L)
#define TOBJ_HIDE	(104L)
#define TOBJ_SHOW	(105L)


/* defines of attrs (101-200) */
#define TOBJ_LEFT	(101L)
#define TOBJ_TOP	(102L)
#define TOBJ_WIDTH	(103L)
#define TOBJ_HEIGHT	(104L)

#define TOBJ_ACTIVEABLE	(105L)
#define TOBJ_ACTIVE	(106L)
#define TOBJ_HIDDEN	(107L)

#define TOBJ_FGCOLOR	(111L)
#define TOBJ_FGCOLOR2	(112L)
#define TOBJ_BGCOLOR	(113L)
#define TOBJ_BGCOLOR2	(114L)
#define TOBJ_BGCHAR	(115L)

#define TOBJ_NEXTOBJ	(121L)
#define TOBJ_PARENTOBJ	(122L)



void tobj_rhide(struct tobj *obj);
void tobj_show(struct tobj *obj);
int tobj_init(struct tobj *parent, struct tobj *obj);
void tobj_free(struct tobj *);
void tobj_garbage(struct tobj *);
void tobj_dorefresh(struct tobj *);
int tobj_doevent(struct tobj *, int keycode);
int tobj_domethod(struct tobj *, long method, long data);
int tobj_isactive(struct tobj *);
void tobj_activate_next(struct tobj *parent);
void tobj_activate_prev(struct tobj *parent);
void tobj_activate_next2(struct tobj *parent);
void tobj_activate(struct tobj *);
unsigned int tobj_getx(struct tobj *);
unsigned int tobj_gety(struct tobj *);


#endif
