#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and methods */
#include "ttxtview.h"
#include "ttextbox.h"
#include "tscroll.h"
#include "miscfunc.h"
#include <string.h>
#include <ctype.h>


#include "prvclass/ttxtview.h"

/* this function only counts lines and maximum number of chars in line */
static void formattext(char *text, int width, int formating, long *lines,
		       long *mchars)
{
	long i = 0;
	char *s;

	*lines = 1;
	*mchars = 0;

	s = text;
	while (*s) {
		if ((((*s) == '\n' || i >= MAXLINE) && formating == SCROLL)
		    || (((*s) == '\n' || i >= width)
			&& (formating == CUT || formating == WRAP
			    || formating == WORD))) {
			/* moving ptr back to beginning of word */
			if (formating == WORD)
				while (!isspace(*s) && s != text)
					s--;

			/* moving ptr to end of line */
			if (formating == WRAP)
				while ((*s) != '\n' && (*s) != '\0')
					s++;

			/* setting maximal line lenght */
			if (*mchars < i)
				*mchars = i;
			(*lines)++;
			i = 0;
		}
		i++;
		s++;
	}

	return;
}

static void ttxtview_update_scrolls(struct tobj *obj)
{
	struct ttxtview *txtview;
	txtview = obj->data;

	tset(txtview->horiz_scroll, TSCROLL_POS,
	     txget( obj, TTEXTBOX_FIRST_CHAR));
	tset(txtview->vert_scroll, TSCROLL_POS,
	     txget( obj, TTEXTBOX_FIRST_LINE));
}
static int ttxtview_init_scrolls(struct tobj *obj )
{
	struct ttxtview *txtview;
	txtview = obj->data;

	if ((txtview->vert_scroll =
	     tscroll_init(obj->parent, obj->x + obj->w , obj->y,
			  obj->h, TSCROLL_VERT_ON))) {
		tset(txtview->vert_scroll, TOBJ_BGCOLOR, obj->bgcolor);
		tset(txtview->vert_scroll, TOBJ_BGCOLOR2, obj->fgcolor2);
		tset(txtview->vert_scroll, TOBJ_FGCOLOR, obj->bgcolor2);
		tset(txtview->vert_scroll, TOBJ_FGCOLOR2, obj->bgcolor2);

		tset(txtview->vert_scroll, TSCROLL_LENGHT,
				txtview->textbox.nlines);
		tset(txtview->vert_scroll, TSCROLL_POS, 0);
	} else
		return 0;

	if ((txtview->horiz_scroll =
	     tscroll_init(obj->parent, obj->x, obj->y + obj->h,
			  obj->w, TSCROLL_VERT_OFF))) {
		tset(txtview->horiz_scroll, TOBJ_BGCOLOR, obj->bgcolor);
		tset(txtview->horiz_scroll, TOBJ_BGCOLOR2, obj->fgcolor2);
		tset(txtview->horiz_scroll, TOBJ_FGCOLOR, obj->bgcolor2);
		tset(txtview->horiz_scroll, TOBJ_FGCOLOR2, obj->bgcolor2);

		tset(txtview->horiz_scroll, TSCROLL_LENGHT,
				txtview->textbox.nchars);
		tset(txtview->horiz_scroll, TSCROLL_POS, 0);
		return 1;
	} else
		return 0;

}
static int ttxtview_set(struct tobj *obj, struct tobj_set *tos)
{
	struct ttxtview *txtview;
	txtview = obj->data;

	switch (tos->attr) {
	case TTXTVIEW_BUFFER:
		if (txtview->buffer)
			free(txtview->buffer);
		if (!(txtview->buffer = strdup((char *) tos->data)))
			tsys_error_exit("ERROR: While setting new buffer\n");
		tset(obj, TTEXTBOX_BUFFER, TPDATA(txtview->buffer));
		ttxtview_update_scrolls(obj);
		break;
		/* mapping TEXTBOX attributes */
	case TTXTVIEW_FIRST_LINE:
	case TTXTVIEW_FIRST_CHAR:
	case TTXTVIEW_FORMATING:
	case TTXTVIEW_TABSIZE:
		if (txtview->textbox_fnc.domethod
				&& (*txtview->textbox_fnc.domethod)
					(obj, TOBJ_SET, TPDATA(tos) ))
			ttxtview_update_scrolls(obj);
		else
			return 0;
		break;
	default:
		if (txtview->textbox_fnc.domethod)
			return (*txtview->textbox_fnc.domethod)
					(obj, TOBJ_SET, TPDATA(tos) );
		else
			return 0;
	}

	return 1;
}

static int ttxtview_get(struct tobj *obj, struct tobj_get *tog)
{
	struct ttxtview *txtview;
	txtview = obj->data;

	switch (tog->attr) {
	case TTXTVIEW_BUFFER:
		*(tog->data) = TPDATA(txtview->buffer);
		break;
	default:
		if (txtview->textbox_fnc.domethod)
			return (*txtview->textbox_fnc.domethod)
					(obj, TOBJ_GET, TPDATA(tog) );
		else
			return 0;
	}

	return 1;
}
int ttxtview_domethod(struct tobj *obj, long method, long data)
{
	struct ttxtview *txtview;
	txtview = (struct ttxtview *) obj->data;

	switch (method) {
	case TOBJ_SET:
		return ttxtview_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return ttxtview_get(obj, (struct tobj_get *) data);
	case TTEXTBOX_SCROLL_LEFT:
	case TTEXTBOX_SCROLL_RIGHT:
	case TTEXTBOX_SCROLL_BEGINLINE:
	case TTEXTBOX_SCROLL_ENDLINE:
	case TTEXTBOX_SCROLL_UP:
	case TTEXTBOX_SCROLL_DOWN:
	case TTEXTBOX_SCROLL_BEGIN:
	case TTEXTBOX_SCROLL_END:
	case TTEXTBOX_SCROLL_PAGEUP:
	case TTEXTBOX_SCROLL_PAGEDOWN:
		if (txtview->textbox_fnc.domethod
				&& (*txtview->textbox_fnc.domethod)
					(obj, method, data ))
			ttxtview_update_scrolls(obj);
		else
			return 0;
		break;
	default:
		if (txtview->textbox_fnc.domethod)
			return (*txtview->textbox_fnc.domethod)
					(obj, method, data );
		else
			return 0;
	}
	return 1;
}

int ttxtview_doevent(struct tobj *obj, int keycode)
{
	struct ttxtview *txtview;
	txtview = (struct ttxtview *) obj->data;

	switch (keycode) {
	case K_LEFT:
		tobj_domethod(obj, TTEXTBOX_SCROLL_LEFT, 0);
		break;
	case K_RIGHT:
		tobj_domethod(obj, TTEXTBOX_SCROLL_RIGHT, 0);
		break;
	case K_CTL_LEFT:
		tobj_domethod(obj, TTEXTBOX_SCROLL_BEGINLINE, 0);
		break;
	case K_CTL_RIGHT:
		tobj_domethod(obj, TTEXTBOX_SCROLL_ENDLINE, 0);
		break;
	case K_UP:
		tobj_domethod(obj, TTEXTBOX_SCROLL_UP, 0);
		break;
	case K_DOWN:
		tobj_domethod(obj, TTEXTBOX_SCROLL_DOWN, 0);
		break;
	case K_PGUP:
		tobj_domethod(obj, TTEXTBOX_SCROLL_PAGEUP, 0);
		break;
	case K_PGDN:
		tobj_domethod(obj, TTEXTBOX_SCROLL_PAGEDOWN, 0);
		break;
	case K_HOME:
		tobj_domethod(obj, TTEXTBOX_SCROLL_BEGIN, 0);
		break;
	case K_END:
		tobj_domethod(obj, TTEXTBOX_SCROLL_END, 0);
		break;

	default:
		if (txtview->textbox_fnc.doevent)
			return (*txtview->textbox_fnc.doevent)
					(obj, keycode);
		else
			/* keycode not used. maybe someone will use it */
			return 0;
	}
	return 1;
}

void ttxtview_cleanup(struct tobj *obj)
{
	struct ttxtview *txtview;
	txtview = (struct ttxtview *) obj->data;

	ttextbox_cleanup( obj );
	tobj_free(txtview->horiz_scroll);
	tobj_free(txtview->vert_scroll);

	if (txtview->buffer &&  *(txtview->buffer)){
		free(txtview->buffer);
		txtview->buffer = NULL;
	}
}

void ttxtview_free(struct tobj *obj)
{
	struct ttxtview *txtview;
	txtview = (struct ttxtview *) obj->data;

	ttxtview_cleanup( obj );
	
	if (txtview)
		free(txtview);

}

int ttxtview_setup(struct tobj *obj, struct tobj_fnc *fnc, char *text)
{
	struct ttxtview *txtview;
	char nomem[] = "ERROR: While allocating memory for ttxtview.\n";
	txtview = (struct ttxtview *) obj->data;

	if (!(txtview->buffer = strdup(text))) {
		tsys_error(nomem);
		return 0;
	} else {
		long lines, mchars;

		formattext(txtview->buffer, obj->w, SCROLL,
			   &lines, &mchars);
		
		if (!(ttextbox_setup(obj,&(txtview->textbox_fnc),
						txtview->buffer,SCROLL) )) 
			return 0;
		
		fnc->free = ttxtview_free;
		fnc->doevent = ttxtview_doevent;
		fnc->domethod = ttxtview_domethod;
		fnc->dorefresh = txtview->textbox_fnc.dorefresh;

		
		if (!ttxtview_init_scrolls( obj ) )
			return 0;
		
	}
	return 1;
}
struct tobj *ttxtview_init(struct tobj *parent, int x, int y, int w, int h,
			   char *text)
{
	struct tobj *obj;
	struct ttxtview *txtview;
	char nomem[] = "ERROR: While allocating memory for ttxtview.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(txtview = calloc(1, sizeof(struct ttxtview)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = txtview;

		obj->x = x;
		obj->y = y;
		/* making space for scrollers */
		obj->h = h - 1;
		obj->w = w - 1;

		/* setting default values */
                obj->fgcolor = TTXTVIEWFG;
                obj->fgcolor2 = TTXTVIEWFG2;
                obj->bgcolor = TTXTVIEWBG;
                obj->bgcolor2 =TTXTVIEWBG2;

		obj->activeable = 1;
		obj->parent = parent;
		if (( ttxtview_setup( obj, &obj->fnc, text ) )){
			tobj_init(parent, obj);
			tobj_dorefresh(obj);
			return obj;
		} else
			return 0;
	}
}
